#include "../../backend/support/logger.h"
#include "../../backend/support/nodes.h"
#include "../../backend/free-tree/free-tree.h"
#include "bison-actions.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void yyerror(const char * string) {
    LogError("Mensaje: '%s' debido a '%s' (linea %d).", string, yytext, yylineno);
}

void InitializeExpression(Expression * expression) {
    expression->leftExpression = NULL;
    expression->rightExpression = NULL;
    expression->derivative = NULL;
    expression->integrate = NULL;
    expression->evaluate = NULL;
    expression->negative = NULL;
    expression->parenthesis = NULL;
}

void InitializeFunction(Function * function) {
    function->expression = NULL;
    function->composite = NULL;
    function->next = NULL;
    function->domain = NULL;
    function->name = NULL;
}

void InitializePrint(Print * print) {
    print->expression = NULL;
    print->string = NULL;
}

void InitializeIf(If * if_st) {
    if_st->boolean = NULL;
    if_st->block = NULL;
    if_st->else_block = NULL;
}

void InitializeWhile(While * while_st) {
    while_st->boolean = NULL;
    while_st->block = NULL;
}

void InitializeFor(For * for_st) {
    for_st->boolean = NULL;
    for_st->block = NULL;
    for_st->init_variable = NULL;
    for_st->action_variable = NULL;
}

void InitializeStatement(Statement * statement) {
    statement->while_st = NULL;
    statement->print = NULL;
    statement->for_st = NULL;
    statement->if_st = NULL;
    statement->integrate = NULL;
    statement->derivative = NULL;
    statement->evaluate = NULL;
    statement->assignVariable = NULL;
    statement->declareVariable = NULL;
    statement->declareFunction = NULL;
    statement->next = NULL;
}

void InitializeIntegrate(Integrate * integrate) {
    integrate->function = NULL;
    integrate->error = NULL;
    integrate->lvalue = NULL;
    integrate->rvalue = NULL;
}

void InitializeDerivative(Derivative * derivative) {
    derivative->function = NULL;
    derivative->value = NULL;
    derivative->grade = NULL;
}

void InitializeEvaluate(Evaluate * evaluate) {
    evaluate->function = NULL;
    evaluate->value = NULL;
}

// PROGRAM

Program * ProgramGrammarAction(Statement * statement) {
    Program * program = malloc(sizeof(Program));
    AddMalloc(program);
    program->statement = statement;
    state.succeed = true;
    state.program = program;
    return program;
}

// STATEMENT

Statement * StatementEvaluateGrammarAction(Evaluate * evaluate) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = EVALUATE_STATEMENT;
    statement->evaluate = evaluate;
    return statement;
}

Statement * StatementIntegrateGrammarAction(Integrate * integrate) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = INTEGRATE_STATEMENT;
    statement->integrate = integrate;
    return statement;
}

Statement * StatementDerivativeGrammarAction(Derivative * derivative) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = DERIVATIVE_STATEMENT;
    statement->derivative = derivative;
    return statement;
}

Statement * StatementIfGrammarAction(If * if_st) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = IF_STATEMENT;
    statement->if_st = if_st;
    return statement;
}

Statement * StatementForGrammarAction(For * for_st) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = FOR_STATEMENT;
    statement->for_st = for_st;
    return statement;
}

Statement * StatementWhileGrammarAction(While * while_st) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = WHILE_STATEMENT;
    statement->while_st = while_st;
    return statement;
}

Statement * StatementPrintGrammarAction(Print * print) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = PRINT_STATEMENT;
    statement->print = print;
    return statement;
}

Statement * StatementDeclareFunctionGrammarAction(Function * function) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = DECLARE_FUNCTION_STATEMENT;
    statement->declareFunction = function;
    return statement;
}

Statement * StatementAssignVariableGrammarAction(AssignVariable * variable) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = ASSIGN_VARIABLE_STATEMENT;
    statement->assignVariable = variable;
    return statement;
}

Statement * StatementDeclareVariableGrammarAction(Variable * variable) {
    Statement * statement = malloc(sizeof(Statement));
    AddMalloc(statement);
    InitializeStatement(statement);
    statement->type = DECLARE_VARIABLE_STATEMENT;
    statement->declareVariable = variable;
    return statement;
}

Statement * StatementsGrammarAction(Statement * stat1, Statement * stat2) {
    stat1->next = stat2;
    return stat1;
}

// EXPRESSION

Expression * ExpressionOperatorAddGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = ADD_EXPRESSION;
    expression->leftExpression = firstExpression;
    expression->rightExpression = secondExpression;
    return expression;
}

Expression * ExpressionOperatorSubGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = SUB_EXPRESSION;
    expression->leftExpression = firstExpression;
    expression->rightExpression = secondExpression;
    return expression;
}

Expression * ExpressionOperatorMulGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = MUL_EXPRESSION;
    expression->leftExpression = firstExpression;
    expression->rightExpression = secondExpression;
    return expression;
}

Expression * ExpressionOperatorDivGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = DIV_EXPRESSION;
    expression->leftExpression = firstExpression;
    expression->rightExpression = secondExpression;
    return expression;
}

Expression * ExpressionOperatorPowGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = POW_EXPRESSION;
    expression->leftExpression = firstExpression;
    expression->rightExpression = secondExpression;
    return expression;
}

Expression * ExpressionEvaluateGrammarAction(Evaluate * evaluate) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = EVALUATE_EXPRESSION;
    expression->evaluate = evaluate;
    return expression;
}

Expression * ExpressionDerivativeGrammarAction(Derivative * derivative) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = DERIVATIVE_EXPRESSION;
    expression->derivative = derivative;
    return expression;
}

Expression * ExpressionIntegrateGrammarAction(Integrate  * integrate) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = INTEGRATE_EXPRESSION;
    expression->integrate = integrate;
    return expression;
}

Expression * ExpressionPositiveConstantGrammarAction(double value) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = NUMBER_EXPRESSION;
    expression->value = value;
    return expression;
}

Expression * ExpressionNegativeGrammarAction(Expression * exp) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = NEGATIVE_EXPRESSION;
    expression->leftExpression = exp;
    return expression;
}

Expression * ExpressionParenthesisGrammarAction(Expression * exp) {
    Expression * expression = malloc(sizeof(Expression));
    AddMalloc(expression);
    InitializeExpression(expression);
    expression->type = PARENTHESIS_EXPRESSION;
    expression->leftExpression = exp;
    return expression;
}

Expression * ExpressionIdGrammarAction(char * id) {
    Expression * expression = malloc(sizeof(Expression));
    InitializeExpression(expression);
    expression->type = ID_EXPRESSION;
    expression->id = malloc(strlen(id) + 1);
    strcpy(expression->id, id);
    AddMalloc(expression->id);
    AddMalloc(expression);
    return expression;
}

Expression * ExpressionsGrammarAction(Expression * exp1, Expression * exp2) {
    exp1->next = exp2;
    return exp1;
}

// DOMAIN

Domain * DomainGreaterOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = GREATER_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainLesserOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainLesserEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainGreaterEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = GREATER_EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainNotEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = NOT_EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = NULL;
    domain->right = secondExpression;
    return domain;
}

Domain * DomainLesserLesserOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression, Expression * thirdExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_LESSER_DOMAIN;
    domain->left = firstExpression;
    domain->mid = secondExpression;
    domain->right = thirdExpression;
    return domain;
}

Domain * DomainLesserEqualLesserOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression, Expression * thirdExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_EQUAL_LESSER_DOMAIN;
    domain->left = firstExpression;
    domain->mid = secondExpression;
    domain->right = thirdExpression;
    return domain;
}

Domain * DomainLesserLesserEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression, Expression * thirdExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_LESSER_EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = secondExpression;
    domain->right = thirdExpression;
    return domain;
}

Domain * DomainLesserEqualLesserEqualOperatorGrammarAction(Expression * firstExpression, Expression * secondExpression, Expression * thirdExpression) {
    Domain * domain = malloc(sizeof(Domain));
    AddMalloc(domain);
    domain->type = LESSER_EQUAL_LESSER_EQUAL_DOMAIN;
    domain->left = firstExpression;
    domain->mid = secondExpression;
    domain->right = thirdExpression;
    return domain;
}

// BOOLEAN

Boolean * BooleanDomainGrammarAction(Domain * domain) {
    Boolean * boolean = malloc(sizeof(Boolean));
    AddMalloc(boolean);
    boolean->type = DOMAIN_BOOLEAN;
    boolean->domain = domain;
    boolean->left = NULL;
    boolean->right = NULL;
    return boolean;
}

Boolean * BooleanAndGrammarAction(Boolean * left, Boolean * right) {
    Boolean  * boolean = malloc(sizeof(Boolean));
    AddMalloc(boolean);
    boolean->type = AND_BOOLEAN;
    boolean->domain = NULL;
    boolean->left = left;
    boolean->right = right;
    return boolean;
}

Boolean * BooleanOrGrammarAction(Boolean * left, Boolean * right) {
    Boolean  * boolean = malloc(sizeof(Boolean));
    AddMalloc(boolean);
    boolean->type = OR_BOOLEAN;
    boolean->domain = NULL;
    boolean->left = left;
    boolean->right = right;
    return boolean;
}

Boolean * NotBooleanAction(Boolean * notBoolean) {
    Boolean * boolean = malloc(sizeof(Boolean));
    AddMalloc(boolean);
    boolean->type = NOT_BOOLEAN;
    boolean->domain = NULL;
    boolean->left = notBoolean;
    boolean->right = NULL;
    return boolean;
}

Boolean * BooleanGrammarAction(Boolean * boolAux) {
    Boolean * boolean = malloc(sizeof(Boolean));
    AddMalloc(boolean);
    boolean->type = BOOLEAN;
    boolean->domain = NULL;
    boolean->left = boolAux;
    boolean->right = NULL;
    return boolean;
}

// FUNCTION

Function * DeclareFunctionGrammarAction(char * id, Function * func) {
    func->name = malloc(strlen(id) + 1);
    AddMalloc(func->name);
    strcpy(func->name, id);
    return func;
}

Function * FunctionsGrammarAction(Function * func1, Function * func2) {  
    func1->next = func2;
    return func1;
}

Function * CompositeFunctionGrammarAction(Composite * composite) {
    Function * function = malloc(sizeof(Function));
    AddMalloc(function);
    InitializeFunction(function);
    function->composite = composite;
    function->type = COMPOSITE_FUNCTION;
    return function;
}

Function * ExpressionFunctionGrammarAction(Expression * expression) {
    Function * function = malloc(sizeof(Function));
    AddMalloc(function);
    InitializeFunction(function);
    function->expression = expression;
    function->type = EXPRESSION_FUNCTION;
    return function;
}

Function * ExpressionDomainFunctionGrammarAction(Expression * expression, Domain * domain) {
    Function * function = malloc(sizeof(Function));
    AddMalloc(function);
    InitializeFunction(function);
    function->expression = expression;
    function->domain = domain;
    function->type = EXPRESSION_DOMAIN_FUNCTION;
    return function;
}

// COMPOSITE

Composite * CompositeIdGrammarAction(char * id1, char * id2) {
    Composite * composite = malloc(sizeof(Composite));
    composite->type = ID_COMPOSITE;
    composite->outter_func = malloc(strlen(id1) + 1);
    strcpy(composite->outter_func, id1),
    composite->inner_func = malloc(strlen(id2) + 1);
    AddMalloc(composite->inner_func);
    AddMalloc(composite->outter_func);
    AddMalloc(composite);
    strcpy(composite->inner_func, id2);
    return composite;
}

Composite * CompositeCompGrammarAction(char * id, Composite * composite) {
    Composite * compFinal = malloc(sizeof(Composite));
    compFinal->type = COMPOSITE_COMPOSITE;
    compFinal->outter_func = malloc(strlen(id) + 1);
    AddMalloc(compFinal->outter_func);
    AddMalloc(compFinal);
    strcpy(compFinal->outter_func, id);
    compFinal->inner_comp = composite;
    return compFinal;
}

// PRINT

Print * PrintExpressionGrammarAction(Expression * expression) {
    Print * print = malloc(sizeof(Print));
    AddMalloc(print);
    print->type = EXPRESSION_PRINT;
    print->expression = expression;
    print->string = NULL;
    return print;
}

Print * PrintStringGrammarAction(char * string) {
    Print * print = malloc(sizeof(Print));
    print->type = STRING_PRINT;
    print->string = malloc(strlen(string) + 1);
    strcpy(print->string, string);
    print->expression = NULL;
    AddMalloc(print->string);
    AddMalloc(print);
    return print;
}

// IF

If * IfBlockGrammarAction(Boolean * boolean, Block * block) {
    If * if_st = malloc(sizeof(If));
    AddMalloc(if_st);
    InitializeIf(if_st);
    if_st->boolean = boolean;
    if_st->block = block;
    return if_st;
}

If * IfElseBlockGrammarAction(Boolean * boolean, Block * block, Block * else_block) {
    If * if_st = malloc(sizeof(If));
    AddMalloc(if_st);
    InitializeIf(if_st);
    if_st->boolean = boolean;
    if_st->block = block;
    if_st->else_block = else_block;
    return if_st;
}

// WHILE

While * WhileBlockGrammarAction(Boolean * boolean, Block * block) {
    While * while_st = malloc(sizeof(While));
    AddMalloc(while_st);
    InitializeWhile(while_st);
    while_st->boolean = boolean;
    while_st->block = block;
    return while_st;
}

// FOR

For * ForBlockGrammarAction(Variable * init_variable, Boolean * boolean, AssignVariable * action_variable, Block * block) {
    For * for_st = malloc(sizeof(For));
    AddMalloc(for_st);
    InitializeFor(for_st);
    for_st->init_variable = init_variable;
    for_st->boolean = boolean;
    for_st->action_variable = action_variable;
    for_st->block = block;
    return for_st;
}

For * ForBlockNoVarGrammarAction(Boolean * boolean, AssignVariable * action_variable, Block * block) {
    For * for_st = malloc(sizeof(For));
    AddMalloc(for_st);
    InitializeFor(for_st);
    for_st->boolean = boolean;
    for_st->action_variable = action_variable;
    for_st->block = block;
    return for_st;
}

// BLOCK

Block * BlockGrammarAction(Statement * statement) {
    Block * block = malloc(sizeof(Block));
    AddMalloc(block);
    block->statement = statement;
    return block;
}

// DERIVATIVE

Derivative * SimpleDerivativeGrammarAction(char * id, Expression * expression) {
    Derivative * derivative = malloc(sizeof(Derivative));
    InitializeDerivative(derivative);
    derivative->function = malloc(strlen(id) + 1);
    strcpy(derivative->function, id);
    derivative->value = expression;
    derivative->grade = malloc(sizeof(int));
    *derivative->grade = 1;
    AddMalloc(derivative->grade);
    AddMalloc(derivative->function);
    AddMalloc(derivative);
    return derivative;
}

Derivative * GradedDerivativeGrammarAction(int grade, char * id, Expression * expression) {
    Derivative * derivative = malloc(sizeof(Derivative));
    InitializeDerivative(derivative);
    derivative->function = malloc(strlen(id) + 1);
    strcpy(derivative->function, id);
    derivative->value = expression;
    derivative->grade = malloc(sizeof(int));
    *derivative->grade = grade;
    AddMalloc(derivative->grade);
    AddMalloc(derivative->function);
    AddMalloc(derivative);
    return derivative;
}

// INTEGRATE

Integrate * IntegrateGrammarAction(char * id, Expression * lExp, Expression * rExp) {
    Integrate * integrate = malloc(sizeof(Integrate));
    InitializeIntegrate(integrate);
    integrate->function = malloc(strlen(id) + 1);
    AddMalloc(integrate->function);
    AddMalloc(integrate);
    strcpy(integrate->function, id);
    integrate->lvalue = lExp;
    integrate->rvalue = rExp;
    return integrate;
}

Integrate * ErroredIntegrateGrammarAction(Integrate * integrate, double error) {
    if (integrate->error != 0) {
        yyerror("ERR operand appears more than once");
        FreeMallocTable();
        exit(EXIT_FAILURE);
    }
    integrate->error = malloc(sizeof(double));
    AddMalloc(integrate->error);
    *integrate->error = error;
    return integrate;
}

// EVALUATE

Evaluate * EvaluateGrammarAction(char * id, Expression * expression) {
    Evaluate * evaluate = malloc(sizeof(Evaluate));
    InitializeEvaluate(evaluate);
    evaluate->function = malloc(strlen(id) + 1);
    AddMalloc(evaluate->function);
    AddMalloc(evaluate);
    strcpy(evaluate->function, id);
    evaluate->value = expression;
    return evaluate;
}

// ASSIGN VARIABLE

AssignVariable * AssignVariableAddGrammarAction(char * id, Expression * expression) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = expression;
    assign->type = ADD_ASSIGN_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableSubGrammarAction(char * id, Expression * expression) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = expression;
    assign->type = SUB_ASSIGN_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableMulGrammarAction(char * id, Expression * expression) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = expression;
    assign->type = MUL_ASSIGN_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableDivGrammarAction(char * id, Expression * expression) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = expression;
    assign->type = DIV_ASSIGN_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableIncGrammarAction(char * id) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = NULL;
    assign->type = INCREMENT_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableDecGrammarAction(char * id) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = NULL;
    assign->type = DECREMENT_EXPRESSION;
    return assign;
}

AssignVariable * AssignVariableGrammarAction(char * id, Expression * expression) {
    AssignVariable * assign = malloc(sizeof(AssignVariable));
    assign->id = malloc(strlen(id) + 1);
    AddMalloc(assign->id);
    AddMalloc(assign);
    strcpy(assign->id, id);
    assign->expression = expression;
    assign->type = ASSIGN_EXPRESSION;
    return assign;
}

// DeclareVariable

Variable * DeclareVariableGrammarAction(char * id) {
    Variable * variable = malloc(sizeof(Variable));
    variable->name = malloc(strlen(id) + 1);
    AddMalloc(variable->name);
    AddMalloc(variable);
    strcpy(variable->name, id);
    variable->value = NULL;
    return variable;
}

Variable * DeclareValuedVariableGrammarAction(char * id, Expression * expression) {
    Variable * variable = malloc(sizeof(Variable));
    variable->name = malloc(strlen(id) + 1);
    AddMalloc(variable->name);
    AddMalloc(variable);
    strcpy(variable->name, id);
    variable->value = expression;
    return variable;
}

// EscapedString

char * EscapedStringGrammarAction(char * str) {
    char * string = malloc(strlen(str) + 1);
    AddMalloc(string);
    strcpy(string, str);
    return string;
}

char * EscapedStringsGrammarAction(char * str1, char * str2) {
    char * string = malloc(strlen(str1) + strlen(str2) + 1);
    AddMalloc(string);
    string[0] = 0;
    strcat(string, str1);
    strcat(string, str2);
    return string;
}
